# gastempt2.r
# (C) 2005 Dieter Menne, http://www.menne-biomed.de
# Revised October 2007
# Adding graphics and diagnostics to the basic fit of
# gastric emptying curves
# PB: Pinheiro/Bates, Mixed-Effect..., Springer 2000
# See http://www.menne-biomed.de/gastempt1.r for more comments
library(nlme) # Mixed effect number crunching
library(lattice) # Trellis plots
options(digits=3) # print 3 significant digits
# Get our standard functions from external file in same folder
source("fitfuncs.r")

# Get Data from comma-separated (csv) file with header
ge = read.table("gastempt.csv",sep=",",header=TRUE)
# Make sure subject are not treated as numbers
ge$subj = as.factor(ge$subj)
# Create new factor <subj>.<treat>
ge$subjtreat = as.factor(paste(ge$subj,ge$treat,sep="."))
# See PB 8.1.3. Function SSEmptLinExp is defined in fitfuncs.r
gelist = nlsList(v~SSEmptLinExp(t,v0,kappa,tempt)|subjtreat,data=ge)
print(gelist)

# compute nlme fit
contr = nlmeControl(pnlsTol=0.5)
ge0.nlme=nlme(gelist,control=contr)
ge0.coef = coef(ge0.nlme) # get coefficients. This will need a minute
print(ge0.coef)

# Using R-typical "apply" instead of a for-loop in gastempt1
# Function t50 is defined in fitfuncs.r
ge0.coef$t50 = apply(ge0.coef,1,t50)

# The shortest version  using "augmented prediction"
ge0.pred = augPred(ge0.nlme,primary=~t)
p = plot(ge0.pred) # Very quick, a bit ugly
print(p)

# A few bells
p = plot(ge0.pred,
        layout=c(2,4), # grid of 2 horizontal, 4 vert
        aspect=0.7,  # aspect of each panel; 1.0 for square
        main="LinExp Gastric Emptying",
        xlab="time (min)",
        ylab="volume (ml)",
        as.table=T # ordered top to bottom
        )
# why we print()
# http://cran.r-project.org/doc/FAQ/R-FAQ.html#Why-do-lattice_002ftrellis-graphics-not-work_003f
print(p)
# Output the plot to a png file
#trellis.device("png",file="gastempt2.png",
#  width=400,height=600,theme="col.whitebg")
print(p)
dev.off() # don't forget to close the device

# Diagnostic plots of residuals
# remove two # for png output
trellis.device("png",file="gastempt2_diag.png",
  width=400,height=600,theme="col.whitebg")

p = plot(ge0.nlme,subjtreat ~resid(.), abline = 0,
    main= "Boxplot Residuals ge0.nlme",aspect=0.7)
print(p,split=c(1,1,1,2),more=TRUE)
# core line: plot(ge0.nlme)
# the rest is decoration
p = plot(ge0.nlme,
    id=0.05,  # mark significant deviations
    idLabels = paste(ge$subjtreat,ge$t,sep=".t="), # add time
    main=  "Residuals ge0.nlme",
    ylab = "residuals (norm.)",
    xlab = "fitted volumes (ml)",
    aspect=0.7,
    cex=0.7,     # force smaller characters
    xlim=c(200,1200), # larger x limits
    ylim = c(-3,3) # symmetric
    )
print(p,split=c(1,2,1,2))

#dev.off() # remove first # for png output



