# gastempt3.r
# (C) 2005 Dieter Menne, http://www.menne-biomed.de
# Comparing Power Exponential fits with LinExp Fits
# PB: Pinheiro/Bates, Mixed-Effect..., Springer 2000
library(nlme)    # Mixed effect number crunching
library(lattice) # Trellis plots
options(digits=3) # print 3 significant digits
# Get our standard functions from external file in same folder
source("fitfuncs.r")

# Get Data from comma-separated (csv) file with header
ge = read.table("gastempt.csv",sep=",",header=TRUE)
# Make sure subject are not treated as numbers
ge$subj = as.factor(ge$subj)
# Create new factor <subj>.<treat>
ge$subjtreat = as.factor(paste(ge$subj,ge$treat,sep="."))
# Functions SSEmptLinExpLog and SSEmptPowExpLog are defined in fitfuncs.r
# The following runs into problems with logarithms of negative values
#gePowlist = nlsList(v~SSEmptPowExp(t,v0,beta,tempt)|subjtreat,data=ge)
# We use a workaround to avoid negative values of kappa, beta and tempt,
# by estimating the logarithm of the coefficients.
# After all nls/nlme calculations are done, we convert the
# revert the log-transform of the coefficients.
# This is similar to the "phenobarbital" example in PB 6.4
gePowLoglist = nlsList(v~SSEmptPowExpLog(t,v0,logbeta,logtempt)|subjtreat,data=ge)
geLinLoglist = nlsList(v~SSEmptLinExpLog(t,v0,logkappa,logtempt)|subjtreat,data=ge)

print(gePowLoglist)
print(geLinLoglist)

# compute nlme fits
contr = nlmeControl(pnlsTol=0.3)
ge3PowLog.nlme=nlme(gePowLoglist,control=contr)
ge3PowLog.coef = coef(ge3PowLog.nlme)
print(ge3PowLog.nlme)
print(ge3PowLog.coef)

ge3LinLog.nlme=nlme(geLinLoglist,control=contr)
ge3LinLog.coef = coef(ge3LinLog.nlme)
print(ge3LinLog.nlme)
print(ge3LinLog.coef)

# Compute t50
ge3LinLog.coef$t50 = apply(ge3LinLog.coef,1,t50)
ge3PowLog.coef$t50 = apply(ge3PowLog.coef,1,t50)
# Find maximum of all t50 for plotting range
t50.max = floor((max(c(ge3LinLog.coef$t50,ge3PowLog.coef$t50))*1.2)/10)*10

# Using "augmented prediction", this time with extrapolation
ge3LinLog.pred = augPred(ge3LinLog.nlme,primary=~t,maximum=t50.max)
levels(ge3LinLog.pred$.type) = c("predlin","original")
ge3PowLog.pred = augPred(ge3PowLog.nlme,primary=~t,maximum=t50.max)
levels(ge3PowLog.pred$.type) = c("predpow","original")
ge3.pred = rbind(ge3LinLog.pred,
                 ge3PowLog.pred[ge3PowLog.pred$.type == "predpow",])

# prepare a few trellis settings to unclutter plot
par.settings = list(plot.symbol = list(col="black",cex=0.5),
          superpose.line = list(col=c("black","blue","red"),
                                lty=1))
key =  list(lines = list(col=c("black","red")),
         text=list(lab=c("LinExp","PowExp"),cex = c(0.6)))

p = plot(ge3.pred,
        layout=c(2,4), aspect=0.7,as.table=T,
        main="LinExp and PowExp Fits",
        xlab="time (min)", ylab="volume (ml)",
        par.settings = par.settings, key=key)
print(p)

# Output the plot to a png file
trellis.device("png",file="gastempt3.png",
  width=400,height=600,theme="col.whitebg")
# Output to a postscript file
#trellis.device("postscript",file="gastempt3.ps",
#  width=1200,height=1800,theme="col.whitebg",horizontal=F)
print(p)
dev.off() # don't forget to close the device

# Diagnostic plots of residuals
# remove two # for png output
#trellis.device("png",file="gastempt3_diag%03d.png",
#  width=400,height=600,theme="col.whitebg")

PlotResids = function(nlmefit,title) {
  p = plot(nlmefit,subjtreat ~resid(.), abline = 0,
     main= paste("Boxplot Residuals",title),aspect=0.7,
     xlim=c(-120,120))
  print(p,split=c(1,1,1,2),more=TRUE)

  print(p,split=c(1,1,1,2),more=TRUE)
  p = plot(nlmefit,
    id=0.05,  # mark significant deviations
    idLabels = paste(ge$subjtreat,ge$t,sep=".t="), # add time
    main =  paste("Residuals ",title),
    ylab = "residuals (norm.)",
    xlab = "fitted volumes (ml)",
    aspect=0.7, cex=0.7,xlim=c(200,1200),ylim = c(-3,3)
    )
  print(p,split=c(1,2,1,2))
}

PlotResids(ge3PowLog.nlme,"PowExp")
PlotResids(ge3LinLog.nlme,"LinExp")

#dev.off() # remove first # for png output



